from dataclasses import dataclass


@dataclass
class Account:
    id: int
    email: str
    password: str
    is_default: bool


@dataclass
class Session:
    id: int
    csrfhw: str
    username: str
    wlanuserip: str
    attribute_uuid: str
    created_at: int



APP_NAME = "nauta"
APP_AUTHOR = "raulodev"
HOST_URL = "https://secure.etecsa.net:8443/"
LOGOUT_URL = "https://secure.etecsa.net:8443/LogoutServlet"
AVAILABLE_TIME_URL = "https://secure.etecsa.net:8443/EtecsaQueryServlet"



# pylint: disable=w0622
import typer
from nauta.commands.account import add, delete, list, default, password, info
from nauta.commands.session import login, logout, time
from nauta.database import initialize_database


initialize_database()


app = typer.Typer()
app.command()(add)
app.command()(delete)
app.command()(list)
app.command()(default)
app.command()(password)
app.command()(info)
app.command()(login)
app.command()(logout)
app.command()(time)

if __name__ == "__main__":
    app()



import typer
from typing_extensions import Annotated
from nauta.database import (
    list_account,
    add_account,
    delete_account,
    update_account,
    update_password,
    get_account,
)
from nauta.secure import encrypt_password, decrypt_password, generate_key


def add(
    correo: Annotated[str, typer.Argument(help="Correo del usuario")],
    password: Annotated[str, typer.Argument(help="Contraseña del usuario")],
):
    """Añade un nuevo usuario"""

    account = get_account(correo)

    if account:
        typer.echo(typer.style(f"Ya existe el usuario: {correo}", fg="yellow"))
    else:
        secure_password = encrypt_password(password, generate_key())
        add_account(correo, secure_password)
        typer.echo(typer.style(f"Usuario agregado: {correo}", fg="green"))


def delete(
    correo: Annotated[str, typer.Argument(help="Correo del usuario")],
):
    """Elimina un usuario"""
    account = get_account(correo)
    if account:
        delete_account(correo)
        typer.echo(typer.style(f"Usuario eliminado: {correo}", fg="green"))

        # Si queda una solo usuario registrado
        # establecerlo como usuario por defecto
        accounts = list_account()

        if len(accounts) == 1:
            update_account(accounts[0].email, True)

    else:
        typer.echo(typer.style(f"No se encontró el usuario: {correo}", fg="yellow"))


def list():  # pylint: disable=W0622
    """Lista todos los usuarios"""
    accounts = list_account()

    if accounts:
        typer.echo(
            typer.style(
                "👥 Usuarios disponibles (✅ indica usuario por defecto):", fg="cyan"
            )
        )

        for index, account in enumerate(accounts, start=1):
            is_default = "✅" if account.is_default else ""
            typer.echo(f"{index}. {account.email} {is_default}")

    else:

        typer.echo(typer.style("No hay usuarios agregados.", fg="yellow"))


def default(
    correo: Annotated[str, typer.Argument(help="Correo del usuario")],
):
    """Establece el usuario por defecto"""

    account = get_account(correo)
    if account:
        update_account(correo, True)
        typer.echo(
            typer.style(
                f'Usuario "{correo}" establecido como usuario por defecto', fg="green"
            )
        )

    else:
        typer.echo(typer.style(f"No se encontró el usuario: {correo}", fg="yellow"))


def password(
    correo: Annotated[str, typer.Argument(help="Correo del usuario")],
    password: Annotated[  # pylint: disable=W0621
        str, typer.Argument(help="Nueva contraseña del usuario")
    ],
):
    """Establece la contraseña del usuario"""

    account = get_account(correo)
    if account:
        secure_password = encrypt_password(password, generate_key())
        update_password(correo, secure_password)
        typer.echo(typer.style("Contraseña actualizada", fg="green"))

    else:
        typer.echo(typer.style(f"No se encontró el usuario: {correo}", fg="yellow"))


def info(
    correo: Annotated[str, typer.Argument(help="Correo del usuario")],
):
    """Muestra información del usuario"""

    account = get_account(correo)
    if account:
        typer.echo(typer.style(f"👤 Información del usuario: {correo}", fg="cyan"))

        passw = decrypt_password(account.password, generate_key())
        is_default = "✅" if account.is_default else "❌"
        hidden_passw = passw[0] + (len(passw) - 1) * "*"

        typer.echo(f"- Correo: {account.email}")
        typer.echo(f"- Contraseña: {hidden_passw}")
        typer.echo(f"- Es usuario por defecto: {is_default}")
    else:
        typer.echo(typer.style(f"No se encontró el usuario: {correo}", fg="yellow"))



import base64
from hashlib import sha256
from nauta.database import get_secret


def generate_key() -> bytes:
    """Generar una clave derivada de un string (clave simple)"""
    secret = get_secret()
    return sha256(secret.encode()).digest()  # Derivar clave fija


def encrypt_password(password: str, key: bytes) -> str:
    """Encriptar un texto"""
    password_bytes = password.encode("utf-8")
    encrypted_bytes = base64.b64encode(xor_bytes(password_bytes, key))
    return encrypted_bytes.decode("utf-8")


def decrypt_password(encrypted_password: str, key: bytes) -> str:
    """Desencriptar un texto"""
    encrypted_bytes = base64.b64decode(encrypted_password.encode("utf-8"))
    decrypted_bytes = xor_bytes(encrypted_bytes, key)
    return decrypted_bytes.decode("utf-8")


def xor_bytes(data: bytes, key: bytes) -> bytes:
    """XOR entre bytes para encriptar/desencriptar"""
    key_repeated = key * (len(data) // len(key)) + key[: len(data) % len(key)]
    return bytes(a ^ b for a, b in zip(data, key_repeated))



import re
import typer
import requests
from bs4 import BeautifulSoup
from requests import ConnectionError, ReadTimeout
from nauta.constants import HOST_URL, LOGOUT_URL, AVAILABLE_TIME_URL
from nauta.database import add_session, get_session, delete_session


class NautaClient(object):

    def __init__(self, correo, password):
        self.correo = correo
        self.password = password

    def login(self):
        """Función para iniciar sesión"""

        try:

            resp = requests.get(HOST_URL, timeout=60)

            soup = BeautifulSoup(resp.text, "html.parser")

            action = soup.find("form", id="formulario")["action"]
            csrfhw = soup.find("input", {"name": "CSRFHW"})["value"]
            wlanuserip = soup.find("input", {"name": "wlanuserip"})["value"]

            resp = requests.post(
                action,
                data={
                    "username": self.correo,
                    "password": self.password,
                    "CSRFHW": csrfhw,
                    "wlanuserip": wlanuserip,
                },
                headers={"content-type": "application/x-www-form-urlencoded"},
                timeout=60,
            )

            if not resp.ok:
                typer.echo(typer.style(resp.reason, fg="red", bold=True))
                return

            if not "online.do" in resp.url:
                soup = BeautifulSoup(resp.text, "html.parser")
                script_text = soup.find_all("script")[-1].get_text()
                match = re.search(r"alert\(\"(?P<reason>[^\"]*?)\"\)", script_text)
                typer.echo(
                    typer.style(
                        match and match.groupdict().get("reason"), fg="red", bold=True
                    )
                )
                return

            m = re.search(r"ATTRIBUTE_UUID=(\w+)", resp.text)

            attribute_uuid = m.group(1) if m else None

            session = get_session()

            if session:
                typer.echo(
                    typer.style(
                        'Cierre la sesión actual con el comando: "nauta logout"',
                        fg="yellow",
                        bold=True,
                    )
                )
                return

            add_session(csrfhw, self.correo, wlanuserip, attribute_uuid)

            typer.echo(
                typer.style(
                    ("Sesión iniciada"),
                    fg="green",
                    bold=True,
                ),
                nl=False,
            )
            available_time = self.available_time()

            if available_time:

                typer.echo(
                    message=typer.style(
                        f": {available_time} tiempo disponible", bold=True
                    ),
                )

        except (ConnectionError, ReadTimeout) :
            typer.echo(
                typer.style(
                    ("No se pudo conectar con el servidor"), fg="red", bold=True
                )
            )

    def logout(self):
        """Función para cerrar sesión"""

        try:
            session = get_session()

            if not session:
                typer.echo(
                    typer.style(
                        'Primero inicie sesión con el comando: "nauta login"',
                        fg="yellow",
                        bold=True,
                    )
                )
                return

            available_time = self.available_time()

            resp = requests.post(
                LOGOUT_URL,
                data={
                    "CSRFHW": session.csrfhw,
                    "username": session.username,
                    "ATTRIBUTE_UUID": session.attribute_uuid,
                    "wlanuserip": session.wlanuserip,
                },
                headers={"content-type": "application/x-www-form-urlencoded"},
                timeout=60,
            )

            if not resp.ok:
                typer.echo(typer.style(resp.reason, fg="red", bold=True))
                return

            if "SUCCESS" not in resp.text.upper():
                typer.echo(typer.style(resp.text[:100], fg="red", bold=True))
                return

            delete_session()

            typer.echo(
                typer.style(
                    ("Sesión cerrada"),
                    fg="green",
                    bold=True,
                ),
                nl=False,
            )

            if available_time:

                typer.echo(
                    message=typer.style(
                        f": {available_time} tiempo restante", bold=True
                    ),
                )

        except (ConnectionError, ReadTimeout):
            typer.echo(
                typer.style(
                    ("No se pudo conectar con el servidor"), fg="red", bold=True
                )
            )

    def available_time(self) -> str:
        """Función para obtener el tiempo disponible"""

        try:

            session = get_session()

            if not session:
                typer.echo(
                    typer.style(
                        'Primero inicie sesión con el comando: "nauta login"',
                        fg="yellow",
                        bold=True,
                    )
                )
                return

            resp = requests.post(
                AVAILABLE_TIME_URL,
                data={
                    "op": "getLeftTime",
                    "ATTRIBUTE_UUID": session.attribute_uuid,
                    "CSRFHW": session.csrfhw,
                    "username": session.username,
                },
                headers={"content-type": "application/x-www-form-urlencoded"},
                timeout=60,
            )

            if not resp.ok:
                typer.echo(typer.style(resp.reason, fg="red", bold=True))
                return

            return resp.text

        except (ConnectionError, ReadTimeout):
            typer.echo(
                typer.style(
                    ("No se pudo conectar con el servidor"), fg="red", bold=True
                )
            )



import typer
from nauta.client import NautaClient
from nauta.database import get_account
from nauta.secure import decrypt_password, generate_key


def login():
    """Inicia sesión en el nauta hogar"""

    account = get_account(is_default=True)

    if account:
        password = decrypt_password(account.password, generate_key())
        nauta_client = NautaClient(correo=account.email, password=password)
        nauta_client.login()

    else:
        typer.echo(typer.style("Establece un usuario por defecto", fg="yellow"))


def logout():
    """cierra la sesión del nauta hogar"""

    account = get_account(is_default=True)

    if account:
        password = decrypt_password(account.password, generate_key())
        nauta_client = NautaClient(correo=account.email, password=password)
        nauta_client.logout()

    else:
        typer.echo(typer.style("Establece un usuario por defecto", fg="yellow"))


def time():
    """Muestra el tiempo disponible"""

    account = get_account(is_default=True)

    if account:
        password = decrypt_password(account.password, generate_key())
        nauta_client = NautaClient(correo=account.email, password=password)
        available_time = nauta_client.available_time()

        if available_time:

            typer.echo(
                typer.style(
                    f"Tiempo disponible: {available_time}", fg="cyan", bold=True
                )
            )

    else:
        typer.echo(typer.style("Establece un usuario por defecto", fg="yellow"))



import os
from typing import List
import sqlite3
import secrets
from appdirs import user_data_dir
from nauta.models import Account, Session
from nauta.constants import APP_NAME, APP_AUTHOR


def get_global_db_path():
    """Directorio estándar para datos de la aplicación"""
    data_dir = user_data_dir(APP_NAME, APP_AUTHOR)
    os.makedirs(data_dir, exist_ok=True)
    return os.path.join(data_dir, "data.db")


def create_conn():

    db_path = get_global_db_path()
    connection = sqlite3.connect(db_path)
    return connection, connection.cursor()


def initialize_database():
    """Crear tabla si no existe"""

    connection, cursor = create_conn()

    cursor.execute(
        """
        CREATE TABLE IF NOT EXISTS accounts (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            email TEXT NOT NULL,
            password TEXT NOT NULL,
            is_default INTEGER NOT NULL
        )
    """
    )

    cursor.execute(
        """
        CREATE TABLE IF NOT EXISTS session (
            id INTEGER PRIMARY KEY CHECK (id = 1),
            csrfhw TEXT NOT NULL,
            username TEXT NOT NULL,
            wlanuserip TEXT NOT NULL,
            attribute_uuid TEXT NOT NULL,
            created_at REAL DEFAULT (CAST(strftime('%f', 'now') AS REAL))
        )
    """
    )

    cursor.execute(
        """
        CREATE TABLE IF NOT EXISTS secret (
            id INTEGER PRIMARY KEY CHECK (id = 1),
            data TEXT NOT NULL
        )
    """
    )

    secret = get_secret()
    if not secret:
        create_secret()

    connection.commit()
    connection.close()


def create_secret():
    """Función para crear la clave secreta"""

    connection, cursor = create_conn()

    secret = secrets.token_hex(16)
    cursor.execute(
        "INSERT INTO secret (data) VALUES (?)",
        (secret,),
    )

    connection.commit()
    connection.close()


def get_secret():
    """Función para obtener la clave secreta"""

    _, cursor = create_conn()

    row = cursor.execute("SELECT data FROM secret").fetchone()

    return row[0] if row else None


def list_account() -> List[Account]:
    """Función para listar las cuentas"""
    _, cursor = create_conn()

    cursor.execute("SELECT * FROM accounts")
    rows = cursor.fetchall()

    return [Account(*row) for row in rows]


def get_account(email: str = None, is_default: bool = None) -> Account:
    """Función para obtener una cuenta"""
    _, cursor = create_conn()

    if email:

        cursor.execute("SELECT * FROM accounts WHERE email = ?", (email,))

    if is_default:

        cursor.execute("SELECT * FROM accounts WHERE is_default = 1")

    row = cursor.fetchone()

    return Account(*row) if row else None


def add_account(email: str, password: str, is_default: bool = True) -> None:
    """Función para agregar una nueva cuenta"""

    connection, cursor = create_conn()

    if is_default:
        accounts = list_account()
        for account in accounts:
            update_account(account.email)

    cursor.execute(
        """
        INSERT INTO accounts (email, password, is_default)
        VALUES (?, ?, ?)
        """,
        (email, password, is_default),
    )

    connection.commit()
    connection.close()


def delete_account(email: str) -> None:
    """Función para eliminar una cuenta"""
    connection, cursor = create_conn()

    cursor.execute("DELETE FROM accounts WHERE email = ?", (email,))

    connection.commit()
    connection.close()


def update_password(email: str, password: str) -> None:
    """Función para actualizar una contraseña de usuario"""

    connection, cursor = create_conn()

    cursor.execute(
        """
        UPDATE accounts
        SET password = ?
        WHERE email = ?
        """,
        (password, email),
    )

    connection.commit()
    connection.close()


def update_account(email: str, is_default: bool = False) -> None:
    """Función para actualizar una cuenta"""

    connection, cursor = create_conn()

    if is_default:
        accounts = list_account()
        for account in accounts:
            update_account(account.email)

    cursor.execute(
        """
        UPDATE accounts
        SET email = ?, is_default = ?
        WHERE email = ?
        """,
        (email, is_default, email),
    )

    connection.commit()
    connection.close()


def add_session(
    csrfhw: str, username: str, wlanuserip: str, attribute_uuid: str
) -> None:
    """Función para agregar una nueva sesión"""

    connection, cursor = create_conn()

    cursor.execute(
        """
        INSERT INTO session (csrfhw, username, wlanuserip, attribute_uuid)
        VALUES (?, ?, ?, ?)
        """,
        (csrfhw, username, wlanuserip, attribute_uuid),
    )

    connection.commit()
    connection.close()


def delete_session() -> None:
    """Función para eliminar una sesión"""

    connection, cursor = create_conn()

    cursor.execute("DELETE FROM session")

    connection.commit()
    connection.close()


def get_session():
    """Función para obtener una sesión"""

    _, cursor = create_conn()

    cursor.execute("SELECT * FROM session WHERE id = 1")
    row = cursor.fetchone()

    return Session(*row) if row else None
